<?php
/**
 * PostFinance Checkout Magento 2
 *
 * This Magento 2 extension enables to process payments with PostFinance Checkout (https://www.postfinance.ch/).
 *
 * @package PostFinanceCheckout_Payment
 * @author customweb GmbH (http://www.customweb.com/)
 * @license http://www.apache.org/licenses/LICENSE-2.0  Apache Software License (ASL 2.0)
 */
namespace PostFinanceCheckout\Payment\Model\Payment\Method;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Event\ManagerInterface;
use Magento\Payment\Gateway\Command\CommandManagerInterface;
use Magento\Payment\Gateway\Command\CommandPoolInterface;
use Magento\Payment\Gateway\Config\ValueHandlerPoolInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectFactory;
use Magento\Payment\Gateway\Validator\ValidatorPoolInterface;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Store\Model\ScopeInterface;
use Psr\Log\LoggerInterface;
use PostFinanceCheckout\Payment\Api\PaymentMethodConfigurationRepositoryInterface;
use PostFinanceCheckout\Payment\Block\Method\Form;
use PostFinanceCheckout\Payment\Block\Method\Info;
use PostFinanceCheckout\Payment\Model\ApiClient;
use PostFinanceCheckout\Payment\Model\Service\Quote\TransactionService;

/**
 * PostFinance Checkout payment method adapter.
 */
class Adapter extends \Magento\Payment\Model\Method\Adapter
{

    const CAPTURE_INVOICE_REGISTRY_KEY = 'postfinancecheckout_payment_capture_invoice';

    /**
     *
     * @var LoggerInterface
     */
    private $logger;

    /**
     *
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     *
     * @var PaymentMethodConfigurationRepositoryInterface
     */
    private $paymentMethodConfigurationRepository;

    /**
     *
     * @var ApiClient
     */
    private $apiClient;

    /**
     *
     * @var TransactionService
     */
    private $transactionService;

    /**
     *
     * @var int
     */
    private $paymentMethodConfigurationId;

    /**
     *
     * @var \PostFinanceCheckout\Payment\Model\PaymentMethodConfiguration
     */
    private $paymentMethodConfiguration;

    /**
     *
     * @param LoggerInterface $logger
     * @param ManagerInterface $eventManager
     * @param ValueHandlerPoolInterface $valueHandlerPool
     * @param PaymentDataObjectFactory $paymentDataObjectFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param PaymentMethodConfigurationRepositoryInterface $paymentMethodConfigurationRepository
     * @param ApiClient $apiClient
     * @param TransactionService $transactionService
     * @param string $code
     * @param int $paymentMethodConfigurationId
     * @param CommandPoolInterface $commandPool
     * @param ValidatorPoolInterface $validatorPool
     * @param CommandManagerInterface $commandExecutor
     */
    public function __construct(LoggerInterface $logger, ManagerInterface $eventManager,
        ValueHandlerPoolInterface $valueHandlerPool, PaymentDataObjectFactory $paymentDataObjectFactory,
        ScopeConfigInterface $scopeConfig,
        PaymentMethodConfigurationRepositoryInterface $paymentMethodConfigurationRepository, ApiClient $apiClient,
        TransactionService $transactionService, $code, $paymentMethodConfigurationId,
        CommandPoolInterface $commandPool = null, ValidatorPoolInterface $validatorPool = null,
        CommandManagerInterface $commandExecutor = null)
    {
        parent::__construct($eventManager, $valueHandlerPool, $paymentDataObjectFactory, $code, Form::class, Info::class,
            $commandPool, $validatorPool, $commandExecutor, $logger);
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
        $this->paymentMethodConfigurationRepository = $paymentMethodConfigurationRepository;
        $this->apiClient = $apiClient;
        $this->transactionService = $transactionService;
        $this->paymentMethodConfigurationId = $paymentMethodConfigurationId;
    }

    /**
     * Gets the ID of the payment method configuration.
     *
     * @return number
     */
    public function getPaymentMethodConfigurationId()
    {
        return $this->paymentMethodConfigurationId;
    }

    /**
     * Gets the payment method configuration.
     *
     * @return \PostFinanceCheckout\Payment\Model\PaymentMethodConfiguration
     */
    public function getPaymentMethodConfiguration()
    {
        if ($this->paymentMethodConfiguration == null) {
            $this->paymentMethodConfiguration = $this->paymentMethodConfigurationRepository->get(
                $this->paymentMethodConfigurationId);
        }
        return $this->paymentMethodConfiguration;
    }

    public function isAvailable(CartInterface $quote = null)
    {
        if ($quote != null && $quote->getGrandTotal() < 0.0001) {
            return false;
        }

        if (! parent::isAvailable($quote)) {
            return false;
        }

        if ($quote != null && $this->apiClient->checkApiClientData()) {
            $spaceId = $this->scopeConfig->getValue('postfinancecheckout_payment/general/space_id',
                ScopeInterface::SCOPE_STORE, $quote->getStoreId());
            if (! empty($spaceId)) {
                try {
                    $possiblePaymentMethods = $this->transactionService->getPossiblePaymentMethods($quote);
                    if (! $this->isPaymentMethodPossible($possiblePaymentMethods)) {
                        return false;
                    }
                } catch (\Exception $e) {
                    $this->logger->critical($e);
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }

        return true;
    }

    /**
     * Gets whether the selected payment method can be used.
     *
     * @param \PostFinanceCheckout\Sdk\Model\PaymentMethodConfiguration[] $possiblePaymentMethods
     * @return boolean
     */
    private function isPaymentMethodPossible(array $possiblePaymentMethods)
    {
        foreach ($possiblePaymentMethods as $possiblePaymentMethod) {
            if ($possiblePaymentMethod->getId() == $this->getPaymentMethodConfiguration()->getConfigurationId()) {
                return true;
            }
        }
        return false;
    }
}